import { ICompletionManager, CompletionConnector, ContextConnector } from '@jupyterlab/completer';
import { ISettingRegistry } from '@jupyterlab/settingregistry';
import { INotebookTracker } from '@jupyterlab/notebook';
import { IFileBrowserFactory } from '@jupyterlab/filebrowser';
import { MainAreaWidget } from '@jupyterlab/apputils';
import { settingsIcon } from '@jupyterlab/ui-components';
import { SettingsWidget } from './widgets/Settings';
import { IMainMenu } from '@jupyterlab/mainmenu';
import { ITranslator } from '@jupyterlab/translation';
import MergeConnector from './connectors/mergeConnector';
import MutableAiConnector from './connectors/mutableaiConnector';
import { PLUGIN_ID, IMutableAI } from './tokens';
import { invoke, invokeNotebook, select, selectNotebook, toggleFlag, updateSettings } from './commands';
import { IDocumentManager } from '@jupyterlab/docmanager';
import { MutableAIManager } from './manager';
const plugin = {
    id: PLUGIN_ID,
    autoStart: true,
    provides: IMutableAI,
    requires: [
        IFileBrowserFactory,
        ISettingRegistry,
        IMainMenu,
        ITranslator,
        INotebookTracker,
        IDocumentManager,
        ICompletionManager
    ],
    activate: (app, factory, settings, mainMenu, translator, notebooks, docManager, completionManager) => {
        const { commands, contextMenu, docRegistry } = app;
        /*
          Initialized main mutableAI manager object.
        */
        const manager = new MutableAIManager({
            translator,
            mainMenu,
            commands,
            contextMenu,
            factory,
            getSettings: () => settings.load(PLUGIN_ID),
            docRegistry,
            docManager: docManager,
            app
        });
        console.log('Mutable AI context menu is activated!');
        let flag = true;
        let apiKey = '';
        let autocompleteDomain = '';
        let transformDomain = '';
        /**
         * Load the settings for this extension
         *
         * @param setting Extension settings
         */
        // This is used to initiate autocomplete.
        const initAutocomplete = (panel) => {
            var _a, _b;
            const settings = {
                flag,
                apiKey,
                autocompleteDomain,
                transformDomain
            };
            let editor = (_b = (_a = panel.content.activeCell) === null || _a === void 0 ? void 0 : _a.editor) !== null && _b !== void 0 ? _b : null;
            const contextConnector = new ContextConnector({ editor });
            const handler = completionManager.register({
                connector: contextConnector,
                editor,
                parent: panel
            });
            const updateConnector = () => {
                var _a, _b;
                editor = (_b = (_a = panel.content.activeCell) === null || _a === void 0 ? void 0 : _a.editor) !== null && _b !== void 0 ? _b : null;
                if (editor) {
                    const mutableaiOptions = {
                        session: panel.sessionContext.session,
                        editor,
                        panel,
                        settings
                    };
                    const options = {
                        editor,
                        session: panel.sessionContext.session,
                        path: panel.context.sessionContext.name
                    };
                    handler.editor = editor;
                    // @ts-ignore
                    const mutableai = new MutableAiConnector(mutableaiOptions);
                    const connector = new CompletionConnector(options);
                    handler.connector = new MergeConnector(mutableai, connector);
                }
            };
            updateConnector();
            panel.content.activeCellChanged.connect(updateConnector);
            panel.sessionContext.sessionChanged.connect(updateConnector);
        };
        // Wait for the application to be restored and
        // for the settings for this plugin to be loaded
        Promise.all([app.restored, settings.load(PLUGIN_ID)]).then(([, setting]) => {
            function loadSetting(setting) {
                // Read the settings and convert to the correct type
                flag = setting.get('flag').composite;
                apiKey = setting.get('apiKey').composite;
                autocompleteDomain = setting.get('autocompleteDomain')
                    .composite;
                transformDomain = setting.get('transformDomain').composite;
                notebooks.forEach(panel => {
                    initAutocomplete(panel);
                });
            }
            // Read the settings
            loadSetting(setting);
            // Listen for your plugin setting changes using Signal
            setting.changed.connect(loadSetting);
            /*
              Mutable AI toggle AutoComplete flag in main menu command.
            */
            commands.addCommand(toggleFlag, {
                label: 'AutoComplete',
                isToggled: () => flag,
                execute: () => {
                    // Programmatically change a setting
                    Promise.all([setting.set('flag', !flag)])
                        .then(() => {
                        const newFlag = setting.get('flag').composite;
                        console.log(`Mutable AI updated flag to '${newFlag ? 'enabled' : 'disabled'}'.`);
                    })
                        .catch(reason => {
                        console.error(`Something went wrong when changing the settings.\n${reason}`);
                    });
                }
            });
            /*
              Mutable AI update settings in main menu command.
            */
            commands.addCommand(updateSettings, {
                label: 'Update Mutable AI Settings',
                execute: () => {
                    const close = () => { var _a; return (_a = app.shell.currentWidget) === null || _a === void 0 ? void 0 : _a.close(); };
                    const content = new SettingsWidget(setting, close);
                    const widget = new MainAreaWidget({ content });
                    widget.title.label = 'MutableAI Settings';
                    widget.title.icon = settingsIcon;
                    app.shell.add(widget, 'main');
                }
            });
            notebooks.restored.then(() => {
                notebooks.forEach(panel => {
                    initAutocomplete(panel);
                });
            });
            notebooks.widgetAdded.connect((sender, panel) => {
                initAutocomplete(panel);
            });
            // Add notebook completer command.
            app.commands.addCommand(invokeNotebook, {
                execute: () => {
                    var _a;
                    const panel = notebooks.currentWidget;
                    if (panel && ((_a = panel.content.activeCell) === null || _a === void 0 ? void 0 : _a.model.type) === 'code') {
                        return app.commands.execute(invoke, {
                            id: panel.id
                        });
                    }
                }
            });
            // Add notebook completer select command.
            app.commands.addCommand(selectNotebook, {
                execute: () => {
                    const id = notebooks.currentWidget && notebooks.currentWidget.id;
                    if (id) {
                        return app.commands.execute(select, { id });
                    }
                }
            });
            // Set enter key for notebook completer select command.
            app.commands.addKeyBinding({
                command: selectNotebook,
                keys: ['Enter'],
                selector: '.jp-Notebook .jp-mod-completer-active'
            });
        });
        return manager;
    }
};
export default plugin;
