import { PromiseDelegate, UUID } from '@lumino/coreutils';
import { MainAreaWidget } from '@jupyterlab/apputils';
import { requestAPI } from './handler';
import { CodeViewerWidget } from './widgets/CodeViewer';
import { notebookIcon } from '@jupyterlab/ui-components';
import { INotification } from 'jupyterlab_toastify';
function sleep(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
function determineFileType(name) {
    const re = /(?:\.([^.]+))?$/;
    const ext = re.exec(name);
    return ext ? ext[1] : null;
}
export class TransformPollingManager {
    constructor(props) {
        this._ready = new PromiseDelegate();
        this._mutableAI = null;
        this._count = 0;
        this._file_path = '';
        this._current_file_path = '';
        this._docManager = props.docManager;
        this._app = props.app;
        this._mutableAI = props.mutableAI;
        this._count = 0;
        this._file_path = '';
        this.startPolling = this.startPolling.bind(this);
        this.acceptFile = this.acceptFile.bind(this);
        this.declineFile = this.declineFile.bind(this);
    }
    /**
     * A promise that resolves when the settings have been loaded.
     */
    get ready() {
        return this._ready.promise;
    }
    async startPolling(file_path, current_file_path) {
        this._file_path = file_path;
        this._current_file_path = current_file_path;
        const ext = determineFileType(current_file_path);
        const dataToSend = { file_path, ext };
        // Background task with progression animation
        this._toasterId = await INotification.inProgress('Transformation Started.');
        const pollingRef = setInterval(async () => {
            this._count += 1;
            try {
                const response = await requestAPI('CHECK_STATUS', {
                    body: JSON.stringify(dataToSend),
                    method: 'POST'
                });
                // -> Update text
                INotification.update({
                    toastId: this._toasterId,
                    message: 'Transformation in progress..'
                });
                if ((response === null || response === void 0 ? void 0 : response.status) === 'finished') {
                    // Open transformed file here.
                    // -> Update text, status and set closing delay (in ms)
                    INotification.update({
                        toastId: this._toasterId,
                        message: 'Transformed successfully!',
                        type: 'success',
                        autoClose: 3000
                    });
                    const close = () => { var _a; return (_a = this._app.shell.currentWidget) === null || _a === void 0 ? void 0 : _a.close(); };
                    const id = UUID.uuid4();
                    const ext = determineFileType(this._current_file_path);
                    const content = new CodeViewerWidget(close, response.file, this._app, {
                        onAcceptChanges: () => this.acceptFile(id),
                        onDeclineChanges: () => this.declineFile(id)
                    }, ext);
                    const file_name_arr = this._current_file_path.split('/');
                    const widget = new MainAreaWidget({ content });
                    widget.title.label = `Code Viewer ${+file_name_arr.length
                        ? '- ' + file_name_arr[file_name_arr.length - 1]
                        : ''}`;
                    widget.id = id;
                    widget.title.icon = notebookIcon;
                    this._app.shell.add(widget, 'main', { mode: 'split-right' });
                    clearInterval(pollingRef);
                }
                else {
                    if (this._count >= 60) {
                        // -> Update text, status and set closing delay (in ms)
                        INotification.update({
                            toastId: this._toasterId,
                            message: 'Transformation failed!',
                            type: 'error',
                            autoClose: 3000
                        });
                        this._count = 0;
                        this._file_path = '';
                        clearInterval(pollingRef);
                    }
                }
            }
            catch (error) {
                // -> Update text, status and set closing delay (in ms)
                INotification.update({
                    toastId: this._toasterId,
                    message: 'Transformation failed!',
                    type: 'error',
                    autoClose: 3000
                });
                console.log('Failed to make request. Reason: ' + error.toString());
            }
        }, 1000);
    }
    async acceptFile(id) {
        var _a, _b;
        const apiKey = (_a = this._mutableAI) === null || _a === void 0 ? void 0 : _a.get('apiKey').composite;
        const transformDomain = (_b = this._mutableAI) === null || _b === void 0 ? void 0 : _b.get('transformDomain').composite;
        const dataToSend = {
            file_path: this._file_path,
            current_file_path: this._current_file_path,
            url: transformDomain,
            action: 'accept',
            apiKey
        };
        const widgets = this._app.shell.widgets('main');
        while (true) {
            const widget = widgets.next();
            if (widget) {
                const widgetContext = this._docManager.contextForWidget(widget);
                if ((widgetContext === null || widgetContext === void 0 ? void 0 : widgetContext.path) === this._current_file_path) {
                    if (widgetContext.model.dirty) {
                        await widgetContext.save();
                        widget.close();
                    }
                    else {
                        widget.close();
                    }
                }
            }
            else {
                break;
            }
        }
        try {
            const response = await requestAPI('FILE_ACTION', {
                body: JSON.stringify(dataToSend),
                method: 'POST'
            });
            if (response.status === 'completed') {
                console.log('File accepted.');
                const widgets = this._app.shell.widgets('main');
                while (true) {
                    const widget = widgets.next();
                    if (widget) {
                        if (widget.id === id) {
                            widget.close();
                            sleep(1000);
                        }
                    }
                    else {
                        break;
                    }
                }
                this._docManager.open(this._current_file_path);
            }
        }
        catch (e) {
            console.log('File accepting failed.', e);
        }
    }
    async declineFile(id) {
        var _a, _b;
        const apiKey = (_a = this._mutableAI) === null || _a === void 0 ? void 0 : _a.get('apiKey').composite;
        const transformDomain = (_b = this._mutableAI) === null || _b === void 0 ? void 0 : _b.get('transformDomain').composite;
        const dataToSend = {
            file_path: this._file_path,
            current_file_path: this._current_file_path,
            action: 'decline',
            url: transformDomain,
            apiKey
        };
        try {
            const response = await requestAPI('FILE_ACTION', {
                body: JSON.stringify(dataToSend),
                method: 'POST'
            });
            if (response.status === 'completed') {
                console.log('File declined.');
                const widgets = this._app.shell.widgets('main');
                while (true) {
                    const widget = widgets.next();
                    if (widget) {
                        if (widget.id === id) {
                            widget.close();
                            break;
                        }
                    }
                    else {
                        break;
                    }
                }
            }
        }
        catch (e) {
            console.log('File declining failed.', e);
        }
    }
}
